Package: grec
Type: Package
Title: Gradient-Based Recognition of Spatial Patterns in Environmental
        Data
Version: 1.6.4
Date: 2026-05-13
URL: https://github.com/LuisLauM/grec
BugReports: https://github.com/LuisLauM/grec/issues
Authors@R: c(
    person(
      given = "Wencheng",
      family = "Lau-Medrano", 
      email = "luis.laum@protonmail.com", 
      role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-7979-9710")
    )
  )
Maintainer: Wencheng Lau-Medrano <luis.laum@protonmail.com>
Description: Provides algorithms for detection of spatial patterns from oceanographic data using image processing methods based on Gradient Recognition.
License: GPL (>= 3)
Depends: R (>= 4.1.0)
Imports: utils, imagine (>= 2.1.2), raster, terra, abind, lifecycle,
        cli
LazyData: true
Encoding: UTF-8
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-13 15:59:59 UTC; afipdbl
Author: Wencheng Lau-Medrano [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7979-9710>)
Repository: CRAN
Date/Publication: 2026-05-14 14:10:16 UTC
Built: R 4.6.0; ; 2026-05-14 15:42:32 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(getGradients,RasterLayer)
S3method(getGradients,SpatRaster)
S3method(getGradients,array)
S3method(getGradients,default)
S3method(getGradients,list)
S3method(getGradients,matrix)
export(detectFronts)
export(getGradients)
importFrom(abind,abind)
importFrom(cli,cli_abort)
importFrom(imagine,agenbagFilters)
importFrom(imagine,contextualMF)
importFrom(imagine,convolution2D)
importFrom(imagine,medianFilter)
importFrom(lifecycle,badge)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(raster,nlayers)
importFrom(raster,raster)
importFrom(raster,values)
importFrom(terra,'values<-')
importFrom(terra,as.array)
importFrom(terra,crs)
importFrom(terra,ext)
importFrom(terra,nlyr)
importFrom(terra,origin)
importFrom(terra,rast)
importFrom(terra,values)
importFrom(terra,varnames)
importFrom(utils,modifyList)
importMethodsFrom(raster,"[")
importMethodsFrom(terra,"ext<-")
importMethodsFrom(terra,"origin<-")
importMethodsFrom(terra,"varnames<-")
importMethodsFrom(terra,as.matrix)

# Detect fronts --------------------------------------------------------------------------
# Load example data
data(sst)
exampleChlData <- list(x = chl$longitude,
                       y = chl$latitude,
                       z = chl$chlorophyll[,,1])

# Apply detectFronts function
out <- detectFronts(x = exampleChlData, intermediate = FALSE)
str(out)

# Make plots -----------------------------------------------------------------------------

xlim <- round(range(exampleChlData$x), 0)
ylim <- round(range(exampleChlData$y), 0)

# Define axis values
xAxis <- seq(xlim[1], xlim[2], 5)
yAxis <- seq(ylim[1], ylim[2], 2)

# Set plot parameters
par(xaxs = "i", yaxs = "i", mar = rep(0, 4), oma = c(2, 3, 2, 1),
    mfrow = c(1, 3))

# Plot original map
image(exampleChlData, axes = FALSE, col = colPalette, xlim = xlim, ylim = ylim)
box()

mtext(text = "Original", side = 3, line = -2, adj = 0.99, cex = 1.2, font = 2)
mtext(text = paste(c("Chlorophyll-a, Aqua MODIS,",
                     "Monthly Composite",
                     "March, 2010",
                     "[85\u00b0 W - 70\u00b0 W] [20\u00b0 S - 0\u00b0 S]"),
                   collapse = "\n"),
      side = 3, line = -8, adj = 0.99)
axis(side = 1, at = xAxis, labels = paste(abs(xAxis), "\u00b0W"))
axis(side = 2, at = yAxis, labels = paste(abs(yAxis), "\u00b0S"), las = 2)

# Plot gradient map
image(out, axes = FALSE, col = colPalette, xlim = xlim, ylim = ylim)
box()

mtext(text = "Gradient from\nBelkin & O'Reilly (2009)", side = 3, line = -4,
      adj = 0.99, cex = 1.2, font = 2)
axis(side = 3, at = xAxis, labels = paste(abs(xAxis), "\u00b0W"))

# Plot gradient map in logaritmic scale
logOut <- out
logOut$z <- log10(logOut$z)
image(logOut, axes = FALSE, col = colPalette, xlim = xlim, ylim = ylim)
box()

mtext(text = "Gradient (log scale) from\nBelkin & O'Reilly (2009)", side = 3,
      line = -4, adj = 0.99, cex = 1.2, font = 2)
axis(side = 1, at = xAxis, labels = paste(abs(xAxis), "\u00b0W"))

#' @rdname getGradients
#' @method getGradients array
#' @export
getGradients.array <- function(x, method = "BelkinOReilly2009",
                               intermediate = FALSE, ...){

  # Check arguments
  checkArgs_df_array(x = x)

  output <- lapply(
    X = seq(dim(x)[3]),
    FUN = \(i, ...) getGradients(x = x[,,i], ...),
    method = method, intermediate = intermediate,
    checkPrevs = FALSE, ...
  )

  if(isTRUE(intermediate)){
    index <- names(output[[1]])

    output <- lapply(
      X = seq_along(index),
      FUN = \(x) lapply(output, "[[", x) |> abind(along = 3)
    )

    names(output) <- index
  }else{
    output <- abind(output, along = 3)

    dimnames(output) <- dimnames(x)
  }

  output
}

#' @title Default color palette most using on environmental representations.
#' @description Vector with 2000 colors generated from `tim.colors` function.
#' @docType data
#' @usage colPalette
#' @format A vector of 2000 colors in RGB format.
#' @references `tim.colors` from **fields** package
"colPalette"

#' @title Sea Surface Temperature Data
#' @description SST maps downloaded from ERDDAP for running examples with
#' `grec` functions.
#' @docType data
#' @usage sst
#' @format A `list` with SST information from February to April of Aqua
#' MODIS source.
#' @references ERDDAP website: <https://coastwatch.pfeg.noaa.gov/erddap>
"sst"

#' @title Sea Surface Chlorophyll Data
#' @description Surface chlorophyll maps downloaded from ERDDAP for running
#' examples with `grec` functions.
#' @docType data
#' @usage chl
#' @format A `list` with chlorophyll information from February to April of
#' Aqua MODIS source.
#' @references ERDDAP website: <https://coastwatch.pfeg.noaa.gov/erddap>
"chl"

#' @rdname getGradients
#' @method getGradients list
#' @export
getGradients.list <- function(x, method = "BelkinOReilly2009",
                              intermediate = FALSE, ...){

  # Check arguments
  checkArgs_df_list(x = x)

  # Apply method
  x$z <- getGradients(
    x = x$z,
    method = method,
    intermediate = intermediate,
    checkPrevs = FALSE,
    ...
  )

  x
}

#' @title GRadient-based RECognition of spatial patterns in environmental data
#' @description Provides algorithms for detecting spatial patterns from 2-D
#' oceanographic data using image processing methods based on Gradient
#' Recognition.
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom abind abind
#' @importFrom cli cli_abort
#' @importFrom imagine agenbagFilters
#' @importFrom imagine contextualMF
#' @importFrom imagine convolution2D
#' @importFrom imagine medianFilter
#' @importFrom lifecycle badge
#' @importFrom lifecycle deprecate_soft
#' @importFrom lifecycle deprecate_warn
#' @importFrom lifecycle deprecated
#' @importFrom raster nlayers
#' @importFrom raster raster
#' @importFrom raster values
#' @importFrom terra 'values<-'
#' @importFrom terra as.array
#' @importFrom terra crs
#' @importFrom terra ext
#' @importFrom terra nlyr
#' @importFrom terra origin
#' @importFrom terra rast
#' @importFrom terra values
#' @importFrom terra varnames
#' @importFrom utils modifyList
#' @importMethodsFrom raster "["
#' @importMethodsFrom terra as.matrix
#' @importMethodsFrom terra ext<-
#' @importMethodsFrom terra origin<-
#' @importMethodsFrom terra varnames<-
## usethis namespace: end
NULL

#' @rdname getGradients
#' @method getGradients RasterLayer
#' @export
getGradients.RasterLayer <- function(x, method = "BelkinOReilly2009",
                                     intermediate = FALSE, ...){

  deprecate_warn(
    when = "as soon as raster package is removed from CRAN",
    what = I("raster::RasterLayer method"),
    with = I("terra::SpatRaster method")
  )
  checkArgs_df_RasterLayer(x = x)

  # Extract coordinates and data for calculate fronts from Raster and convert to list
  startMatrix <- values(x)
  startMatrix <- list(
    x = seq(from = x@extent@xmin, to = x@extent@xmax, length.out = x@ncols),
    y = seq(from = x@extent@ymin, to = x@extent@ymax, length.out = x@nrows),
    z = matrix(data = startMatrix, nrow = x@ncols)
  )

  allOuts <-  getGradients(
    x = startMatrix,
    method = method,
    intermediate = intermediate,
    checkPrevs = FALSE,
    ...
  )

  # Depending on 'intermediate', the output will be a single Raster or a list of them
  if(intermediate){
    allSteps <- dimnames(allOuts$z)[[3]]

    output <- list()
    for(i in seq_along(allSteps)){
      tempOut <- x
      tempOut[] <- as.numeric(allOuts$z[,,i])
      output[[i]] <- tempOut
    }

    names(output) <- allSteps
  }else{
    output <- x
    output[] <- as.numeric(allOuts$z)
  }

  output
}

#' @rdname getGradients
#' @export
getGradients.default <- function(x, method = "BelkinOReilly2009",
                                 intermediate = FALSE,
                                 ConvolNormalization = FALSE, ...){

  # Decide the method
  switch(simplifyChars(x = method),
         belkinoreilly2009 = df_BOR(a = x,
                                    intermediate = intermediate,
                                    ConvNorm = ConvolNormalization,
                                    ...),
         medianfilter = df_MF(x = x,
                               intermediate = intermediate,
                               ConvNorm = ConvolNormalization,
                               ...),
         agenbag20031 = df_Agenbag(x = x,
                                   intermediate = intermediate,
                                   ConvNorm = ConvolNormalization,
                                   algorithm = 1,
                                   ...),
         agenbag20032 = df_Agenbag(x = x,
                                   intermediate = intermediate,
                                   algorithm = 2,
                                   ...))
}

# Names of objects in this function follows the description of the algorithm in
# the article of Belkin and O'Reilly
df_BOR <- function(a, intermediate, ConvNorm, ...){

  # Define default kernel values (weights)
  control_default <- list(kernelValues = c(-1, -2, -1, 0, 0, 0, 1, 2, 1))

  # Merging with extra arguments passed from ...
  extraParams <- modifyList(x = control_default, val = list(...))

  # Apply a smooth (Contextual Median Filter)
  A <- contextualMF(X = a)

  # Define sobel kernel values
  sobelKernel <- extraParams$kernelValues

  # Define sobel kernels
  GX <- matrix(data = sobelKernel, nrow = 3)
  GY <- apply(t(GX), 2, rev) # GX rotated 90 deg counter-clockwise

  # Apply sobel filters (horizontal and verticaly)
  Gx <- convolution2D(X = A, kernel = GX)
  Gy <- convolution2D(X = A, kernel = GY)

  # Apply IDL normalization
  if(ConvNorm){
    Gx <- Gx/sum(abs(sobelKernel), na.rm = TRUE)
    Gy <- Gy/sum(abs(sobelKernel), na.rm = TRUE)
  }

  # Calculate gradient magnitude
  GM <- sqrt(Gx^2 + Gy^2)

  # Return output
  if(intermediate){
    list(original = a,
         CMF      = A,
         Gx       = Gx,
         Gy       = Gy,
         GD       = atan(Gy/Gx), # Gradient direction
         GM       = GM)          # Gradient magnitude
  }else{
    GM
  }
}

df_MF <- function(x, intermediate, ConvNorm, ...){

  # Define default kernel values (weights), radius and times for filter application
  control_default <- list(radius = 3,
                          times = 1,
                          kernelValues = c(-1, -2, -1, 0, 0, 0, 1, 2, 1))

  # Merging with extra arguments passed from ...
  extraParams <- modifyList(x = control_default, val = list(...))

  # Apply a smooth (Median Filter)
  preMatrix <- medianFilter(X = x, radius = extraParams$radius,
                            times = extraParams$times)

  # Define sobel kernel values
  sobelKernel <- extraParams$kernelValues

  # Define sobel kernels
  sobelV <- matrix(data = sobelKernel, nrow = 3)
  sobelH <- apply(t(sobelV), 2, rev) # sobelV rotated 90 deg counter-clockwise

  # Apply sobel filters (horizontal and verticaly)
  filteredV <- convolution2D(X = preMatrix, kernel = sobelV)
  filteredH <- convolution2D(X = preMatrix, kernel = sobelH)

  # Apply IDL normalization
  if(ConvNorm){
    filteredV <- filteredV/sum(abs(sobelKernel), na.rm = TRUE)
    filteredH <- filteredH/sum(abs(sobelKernel), na.rm = TRUE)
  }

  # Calculate gradient
  newSobel <- sqrt(filteredV^2 + filteredH^2)

  # Return output
  if(intermediate){
    list(original           = x,
         median_filter      = preMatrix,
         filtered_v         = filteredV,
         filtered_h         = filteredH,
         gradient_magnitude = newSobel,
         gradient_direction = atan(filteredH/filteredV))
  }else{
    newSobel
  }
}

df_Agenbag <- function(x, intermediate, algorithm, ...){

  # Apply a smooth (Median Filter)
  gradientMat <- agenbagFilters(X = x, algorithm = algorithm)

  # Return output
  if(intermediate){
    list(original           = x,
         gradient_magnitude = gradientMat)
  }else{
    gradientMat
  }
}

#' @rdname getGradients
#' @method getGradients matrix
#' @export
getGradients.matrix <- function(x, method = "BelkinOReilly2009",
                                intermediate = FALSE, ...){

  # Check arguments
  checkArgs_df_matrix(x = x)

  # Apply method
  output <- getGradients.default(
    x = x,
    method = method,
    intermediate = intermediate,
    ...
  )

  dimnames(output) <- dimnames(x)

  output
}

#' @title Apply gradient-based methodologies to environmental data
#'
#' @param ... Same arguments than \link[grec]{getGradients}.
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' This function empowers users to analyze data from various sources,
#' including numeric `matrix`, `array`s, XYZ-`list`s,
#' `SpatRaster`s, or `RasterLayer`s*, by applying gradient-seeking
#' methodologies.
#'
#' @details
#' Since version 1.6.0, this function has been entirely replaced by
#' \link[grec]{getGradients}. As of version 2.0.0, `detectFronts` will no longer
#' be available.
#'
#' @export
detectFronts <- function(...){

  deprecate_soft(when = "1.6.0", what = "detectFronts()", with = "getGradients()")

  getGradients(...)
}

checkArgs_df_matrix <- function(x){
  # Check if x is a valid numerical matrix
  if(!is.numeric(x)){
    cli_abort(
      message = c(
        "{.code x} must be a numeric matrix with environmental data.",
        "i" = "See {.run help(getGradients)}."
      )
    )
  }

  # Check if x is a valid numerical matrix
  if(sum(is.na(as.numeric(x))) == prod(dim(x))){
    cli_abort(
      message = c(
        "There is not any valid values for {.code x}."
      )
    )
  }

  invisible()
}

checkArgs_df_array <- function(x){
  # Check if x is a valid numerical matrix
  if(!is.numeric(x)){
    if(!is.numeric(x)){
      cli_abort(
        message = c(
          "{.code x} must be a numeric array with environmental data.",
          "i" = "See {.run help(getGradients)}."
        )
      )
    }
  }

  # Check if x is a valid numerical matrix
  if(sum(is.na(as.numeric(x))) == prod(dim(x))){
    cli_abort(
      message = c(
        "There is not any valid values for {.code x}."
      )
    )
  }

  invisible()
}

checkArgs_df_list <- function(x){
  # Check if x is a list with 'x', 'y', 'z' dimensions, where z is a numeric matrix/array
  index <- (length(x) == 3 && all(is.element(c("x", "y", "z"), names(x))) && is.numeric(x$x) && is.numeric(x$y) &&
              (is.matrix(x$z) || is.array(x$z)) && is.numeric(x$z))
  if(!index){
    cli_abort(
      message = c(
        "{.code x} must be a XYZ list containing environmental map info (whether a matrix or an array).",
        "i" = "See {.run help(getGradients)}."
      )
    )
  }

  invisible()
}

checkArgs_df_RasterLayer <- function(x){

  if(nlayers(x) < 1){
    cli_abort(
      message = c(
        "RasterLayer object must have at least 1 layer."
      )
    )
  }
  invisible()
}

checkArgs_df_SpatRaster <- function(x){

  if(nlyr(x) < 1){
    cli_abort(
      message = c(
        "SpatRaster object must have at least 1 layer."
      )
    )
  }

  invisible()
}

simplifyChars <- function(x) tolower(gsub(x = x, pattern = "[[:punct:]]", replacement = ""))

checkArgs_prevs <- function(allArgs, type){

  # Define parameters
  method <- simplifyChars(allArgs$method)
  intermediate <- allArgs$intermediate
  ConvolNormalization <- allArgs$ConvolNormalization

  # Check name of method
  methodList <- c("belkinoreilly2009", "medianfilter", "agenbag20031", "agenbag20032")
  if(!is.element(method, methodList)){
    cli_abort(
      message = c(
        "No valid method for gradient calculation."
      )
    )
  }

  # Check 'intermediate'
  if(length(intermediate) != 1 || !is.logical(intermediate)){
    cli_abort(
      message = c(
        "{.code intermediate} must be a single logical value."
      )
    )
  }

  # Check 'ConvolNormalization'
  if(length(ConvolNormalization) != 1 || !is.logical(ConvolNormalization)){
    cli_abort(
      message = c(
        "{.code ConvolNormalization} must be a single logical value."
      )
    )
  }

  invisible()
}

#' @title Apply gradient-based methodologies to environmental data
#'
#' @description This function empowers users to analyze data from various sources,
#' including numeric `matrix`, `array`s, XYZ-`list`s,
#' `SpatRaster`s, or `RasterLayer`s*, by applying gradient-seeking
#' methodologies.
#'
#' @rdname getGradients
#'
#' @param x An object of class `matrix`, `array`, XYZ `list`,
#' `SpatRaster` or `RasterLayer`*. See 'Details.'
#' @param method `character` string indicating the method that will be used.
#' For the available methods, see 'Details'.
#' @param intermediate `logical` indicating whether to get the intermediate
#' matrices (`TRUE`) or just the final one (`FALSE`).
#' @param ConvolNormalization `logical` indicating if convolutions will
#' perform a previous normalization (`FALSE` by default). See Details.
#' @param ... Extra arguments that will depend on the selected method. See
#' Details.
#'
#' @details
#' The \pkg{grec} package collaborates with the \pkg{imagine} package to execute
#' and apply image processing algorithms for identifying oceanic gradients.
#' \pkg{imagine} furnishes the foundational algorithms, developed efficiently
#' utilizing C++ tools. Conversely, \pkg{grec} oversees the utilization of these
#' coding instruments in the context of oceanic gradient recognition and handles
#' the development of input/output methods. In this context, the available methods
#' offered by \pkg{grec} are contingent on the installed \pkg{grec}-\pkg{imagine}
#' versions.
#'
#' (*) Due to the deprecation of the \pkg{raster} package, \pkg{grec} will not be
#' supporting the use of `RasterLayer` in future versions. Instead,
#' we encourage the use of [SpatRaster-class][terra::SpatRaster] (from the
#' \pkg{terra} package), a more recent and actively developed method for working
#' with raster data. This change will take effect as soon as \pkg{raster} is
#' removed from CRAN.
#'
#' Until the current version, `grec` performs four methods:
#' \enumerate{
#' \item `BelkinOReilly2009` (default): Based on Belkin & O'Reilly (2009)
#' article, it uses a Contextual Median Filter (CMF) for smoothing the original
#' data before the applying of Sobel filters.
#' \item `median_filter`: it uses a typical median filter (MF) for
#' smoothing the original data. It also allows the user to change the window
#' size for median filter (3 as default).
#' \item `Agenbag2003-1`: Performs method 1 described on Agenbag et al.
#' (2003) paper, based on the equation:
#' \deqn{SST_{grad}=\sqrt{(T_{i+1}-T_{i-1})^2 +(T_{j+1}-T_{j-1})^2}}
#' \item `Agenbag2003-2`: Performs method 2 described on Agenbag et al.
#' (2003) paper, calculating the the standard deviation of the 3x3 neighbor area
#' for each pixel.
#' }
#'
#' The input data `x` can be represented in various formats to accommodate
#' different data sources. It can be provided as a single numeric `matrix`
#' extracted from an environmental map. Alternatively, it can be represented as
#' a three-dimensional XYZ `list`, where `X` contains a vector of
#' longitudes, `Y` contains a vector of latitudes, and `Z` is a matrix
#' of dimensions `length(x$X)` x `length(x$Y)`. Additionally, it can
#' be specified as an array, `SpatRaster`, or `RasterLayer`* object.
#' If `x` is an `array`, it must have three dimensions: longitude (lon),
#' latitude (lat), and time. It is not mandatory to define the dimnames. The
#' output will maintain all the attributes of the input data.
#'
#'
#' `...` allows the (advanced) users to modify some aspects of filter
#' application. Depending on the selected methodology, some parameters can be
#' modified:
#'
#' \describe{
#' \item{**times**}{`numeric`. How many times do you want to apply
#' the filtering method?}
#' \item{**kernelValues**}{A `numeric` vector that will be used for
#' convolution to detect vertical and horizontal gradients.}
#' \item{**radius**}{`numeric`. If **median-filter** method was
#' selected, it allows to change the window size of the filter.}
#' }
#'
#' Normalization is a standard practice in convolution to maintain the range of
#' output values consistent with the input data. This is achieved by dividing
#' the convolution output by the absolute value of the kernel. While
#' normalization is recommended to ensure consistent interpretation of results,
#' it is disabled by default and can be enabled by setting the
#' `ConvolNormalization` parameter to `TRUE`.
#'
#' Finally, Belkin & O'Reilly's work suggests applying a logarithmic
#' transformation to the gradient output. This step is not enabled by default,
#' as it is primarily intended for chlorophyll data. Users are free to apply the
#' transformation manually if it suits their specific needs.
#'
#' @returns
#' The output class will depend on the input (i.e. the \code{x} argument). For
#' further details about structure of `SpatRaster` or `RasterLayer`, you can
#' check the specific documentation in [terra][terra::SpatRaster] and
#' [raster][raster::raster] respectively.
#'
#'
#' @references
#' Belkin, I. M., & O'Reilly, J. E. (2009). An algorithm for oceanic
#' front detection in chlorophyll and SST satellite imagery. Journal of Marine
#' Systems, 78(3), 319-326 (\doi{10.1016/j.jmarsys.2008.11.018}).
#'
#' Agenbag, J.J., A.J. Richardson, H. Demarcq, P. Freon, S. Weeks,
#' and F.A. Shillington. "Estimating Environmental Preferences of South African
#' Pelagic Fish Species Using Catch Size- and Remote Sensing Data". Progress in
#' Oceanography 59, No 2-3 (October 2003): 275-300.
#' (\doi{https://doi.org/10.1016/j.pocean.2003.07.004}).
#'
#' @export
#'
#' @examples
#' data(sst)
#' exampleSSTData <- list(x = sst$longitude,
#'                        y = sst$latitude,
#'                        z = sst$sst[,,1])
#'
#' data(chl)
#' exampleChlData <- list(x = chl$longitude,
#'                        y = chl$latitude,
#'                        z = chl$chlorophyll[,,1])
#'
#' # Simple application (over a XYZ list)
#' out_sst <- getGradients(x = exampleSSTData)
#' out_chl <- getGradients(x = exampleChlData)
#'
#' # External transformation for chl data
#' out_chl$z <- log10(out_chl$z)
#'
#' oldPar <- par(no.readonly = FALSE)
#'
#' par(mfrow = c(2, 2), mar = rep(0, 4), oma = rep(0, 4))
#'
#' image(exampleSSTData, col = colPalette, axes = FALSE)
#' mtext(text = "Original SST", side = 3, line = -2, adj = 0.99, cex = 1.2)
#'
#' image(out_sst, col = colPalette, axes = FALSE)
#' mtext(text = "SST gradient", side = 3, line = -2, adj = 0.99, cex = 1.2)
#'
#' image(exampleChlData, col = colPalette, axes = FALSE)
#' mtext(text = "Original Chlorophyll", side = 3, line = -2, adj = 0.99, cex = 1.2)
#'
#' image(out_chl, col = colPalette, axes = FALSE)
#' mtext(text = "Chlorophyll gradient\n(log scale)", side = 3, line = -4, adj = 0.99,
#'       cex = 1.2)
#'
#' par(oldPar)
getGradients <- function(x,
                         method = c("BelkinOReilly2009", "median_filter",
                                    "Agenbag2003-1", "Agenbag2003-2"),
                         intermediate = FALSE,
                         ConvolNormalization = FALSE, ...){

  checkPrevs <- list(...)$checkPrevs
  if(is.null(checkPrevs) || checkPrevs){
    # Check and validation of arguments
    checkedArgs <- list(
      method = method[1],
      intermediate = intermediate,
      ConvolNormalization = ConvolNormalization, ...
    )

    checkArgs_prevs(allArgs = checkedArgs, type = class(x))
  }

  UseMethod(generic = "getGradients", object = x)
}

#' @rdname getGradients
#' @method getGradients SpatRaster
#' @export
getGradients.SpatRaster <- function(x, method = "BelkinOReilly2009",
                                    intermediate = FALSE, ...){

  checkArgs_df_SpatRaster(x = x)

  out <- apply(
    X = as.array(x = x),
    MARGIN = 3,
    FUN = getGradients,
    method = method,
    intermediate = FALSE,
    checkPrevs = FALSE,
    simplify = FALSE
  ) |>

    lapply(FUN = t) |> lapply(FUN = as.numeric)

  values(x) <- do.call(args = out, what = c)
  varnames(x) <- sprintf("gradient of %s", varnames(x))

  x
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grec-main.R
\docType{data}
\name{sst}
\alias{sst}
\title{Sea Surface Temperature Data}
\format{
A \code{list} with SST information from February to April of Aqua
MODIS source.
}
\usage{
sst
}
\description{
SST maps downloaded from ERDDAP for running examples with
\code{grec} functions.
}
\references{
ERDDAP website: \url{https://coastwatch.pfeg.noaa.gov/erddap}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grec-main.R
\docType{data}
\name{colPalette}
\alias{colPalette}
\title{Default color palette most using on environmental representations.}
\format{
A vector of 2000 colors in RGB format.
}
\usage{
colPalette
}
\description{
Vector with 2000 colors generated from \code{tim.colors} function.
}
\references{
\code{tim.colors} from \strong{fields} package
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grec-package.R
\docType{package}
\name{grec-package}
\alias{grec}
\alias{grec-package}
\title{GRadient-based RECognition of spatial patterns in environmental data}
\description{
Provides algorithms for detecting spatial patterns from 2-D
oceanographic data using image processing methods based on Gradient
Recognition.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/LuisLauM/grec}
  \item Report bugs at \url{https://github.com/LuisLauM/grec/issues}
}

}
\author{
\strong{Maintainer}: Wencheng Lau-Medrano \email{luis.laum@protonmail.com} (\href{https://orcid.org/0000-0002-7979-9710}{ORCID})

Authors:
\itemize{
  \item Wencheng Lau-Medrano \email{luis.laum@protonmail.com} (\href{https://orcid.org/0000-0002-7979-9710}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detectFronts.R
\name{detectFronts}
\alias{detectFronts}
\title{Apply gradient-based methodologies to environmental data}
\usage{
detectFronts(...)
}
\arguments{
\item{...}{Same arguments than \link[grec]{getGradients}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function empowers users to analyze data from various sources,
including numeric \code{matrix}, \code{array}s, XYZ-\code{list}s,
\code{SpatRaster}s, or \code{RasterLayer}s*, by applying gradient-seeking
methodologies.
}
\details{
Since version 1.6.0, this function has been entirely replaced by
\link[grec]{getGradients}. As of version 2.0.0, \code{detectFronts} will no longer
be available.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Raster-class.R, R/SpatRaster-class.R,
%   R/array-class.R, R/detectFronts-methods.R, R/getGradients.R,
%   R/list-class.R, R/matrix-class.R
\name{getGradients.RasterLayer}
\alias{getGradients.RasterLayer}
\alias{getGradients.SpatRaster}
\alias{getGradients.array}
\alias{getGradients.default}
\alias{getGradients}
\alias{getGradients.list}
\alias{getGradients.matrix}
\title{Apply gradient-based methodologies to environmental data}
\usage{
\method{getGradients}{RasterLayer}(x, method = "BelkinOReilly2009", intermediate = FALSE, ...)

\method{getGradients}{SpatRaster}(x, method = "BelkinOReilly2009", intermediate = FALSE, ...)

\method{getGradients}{array}(x, method = "BelkinOReilly2009", intermediate = FALSE, ...)

\method{getGradients}{default}(
  x,
  method = "BelkinOReilly2009",
  intermediate = FALSE,
  ConvolNormalization = FALSE,
  ...
)

getGradients(
  x,
  method = c("BelkinOReilly2009", "median_filter", "Agenbag2003-1", "Agenbag2003-2"),
  intermediate = FALSE,
  ConvolNormalization = FALSE,
  ...
)

\method{getGradients}{list}(x, method = "BelkinOReilly2009", intermediate = FALSE, ...)

\method{getGradients}{matrix}(x, method = "BelkinOReilly2009", intermediate = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{matrix}, \code{array}, XYZ \code{list},
\code{SpatRaster} or \code{RasterLayer}*. See 'Details.'}

\item{method}{\code{character} string indicating the method that will be used.
For the available methods, see 'Details'.}

\item{intermediate}{\code{logical} indicating whether to get the intermediate
matrices (\code{TRUE}) or just the final one (\code{FALSE}).}

\item{...}{Extra arguments that will depend on the selected method. See
Details.}

\item{ConvolNormalization}{\code{logical} indicating if convolutions will
perform a previous normalization (\code{FALSE} by default). See Details.}
}
\value{
The output class will depend on the input (i.e. the \code{x} argument). For
further details about structure of \code{SpatRaster} or \code{RasterLayer}, you can
check the specific documentation in \link[terra:SpatRaster]{terra} and
\link[raster:raster]{raster} respectively.
}
\description{
This function empowers users to analyze data from various sources,
including numeric \code{matrix}, \code{array}s, XYZ-\code{list}s,
\code{SpatRaster}s, or \code{RasterLayer}s*, by applying gradient-seeking
methodologies.
}
\details{
The \pkg{grec} package collaborates with the \pkg{imagine} package to execute
and apply image processing algorithms for identifying oceanic gradients.
\pkg{imagine} furnishes the foundational algorithms, developed efficiently
utilizing C++ tools. Conversely, \pkg{grec} oversees the utilization of these
coding instruments in the context of oceanic gradient recognition and handles
the development of input/output methods. In this context, the available methods
offered by \pkg{grec} are contingent on the installed \pkg{grec}-\pkg{imagine}
versions.

(*) Due to the deprecation of the \pkg{raster} package, \pkg{grec} will not be
supporting the use of \code{RasterLayer} in future versions. Instead,
we encourage the use of \link[terra:SpatRaster]{SpatRaster-class} (from the
\pkg{terra} package), a more recent and actively developed method for working
with raster data. This change will take effect as soon as \pkg{raster} is
removed from CRAN.

Until the current version, \code{grec} performs four methods:
\enumerate{
\item \code{BelkinOReilly2009} (default): Based on Belkin & O'Reilly (2009)
article, it uses a Contextual Median Filter (CMF) for smoothing the original
data before the applying of Sobel filters.
\item \code{median_filter}: it uses a typical median filter (MF) for
smoothing the original data. It also allows the user to change the window
size for median filter (3 as default).
\item \code{Agenbag2003-1}: Performs method 1 described on Agenbag et al.
(2003) paper, based on the equation:
\deqn{SST_{grad}=\sqrt{(T_{i+1}-T_{i-1})^2 +(T_{j+1}-T_{j-1})^2}}
\item \code{Agenbag2003-2}: Performs method 2 described on Agenbag et al.
(2003) paper, calculating the the standard deviation of the 3x3 neighbor area
for each pixel.
}

The input data \code{x} can be represented in various formats to accommodate
different data sources. It can be provided as a single numeric \code{matrix}
extracted from an environmental map. Alternatively, it can be represented as
a three-dimensional XYZ \code{list}, where \code{X} contains a vector of
longitudes, \code{Y} contains a vector of latitudes, and \code{Z} is a matrix
of dimensions \code{length(x$X)} x \code{length(x$Y)}. Additionally, it can
be specified as an array, \code{SpatRaster}, or \code{RasterLayer}* object.
If \code{x} is an \code{array}, it must have three dimensions: longitude (lon),
latitude (lat), and time. It is not mandatory to define the dimnames. The
output will maintain all the attributes of the input data.

\code{...} allows the (advanced) users to modify some aspects of filter
application. Depending on the selected methodology, some parameters can be
modified:

\describe{
\item{\strong{times}}{\code{numeric}. How many times do you want to apply
the filtering method?}
\item{\strong{kernelValues}}{A \code{numeric} vector that will be used for
convolution to detect vertical and horizontal gradients.}
\item{\strong{radius}}{\code{numeric}. If \strong{median-filter} method was
selected, it allows to change the window size of the filter.}
}

Normalization is a standard practice in convolution to maintain the range of
output values consistent with the input data. This is achieved by dividing
the convolution output by the absolute value of the kernel. While
normalization is recommended to ensure consistent interpretation of results,
it is disabled by default and can be enabled by setting the
\code{ConvolNormalization} parameter to \code{TRUE}.

Finally, Belkin & O'Reilly's work suggests applying a logarithmic
transformation to the gradient output. This step is not enabled by default,
as it is primarily intended for chlorophyll data. Users are free to apply the
transformation manually if it suits their specific needs.
}
\examples{
data(sst)
exampleSSTData <- list(x = sst$longitude,
                       y = sst$latitude,
                       z = sst$sst[,,1])

data(chl)
exampleChlData <- list(x = chl$longitude,
                       y = chl$latitude,
                       z = chl$chlorophyll[,,1])

# Simple application (over a XYZ list)
out_sst <- getGradients(x = exampleSSTData)
out_chl <- getGradients(x = exampleChlData)

# External transformation for chl data
out_chl$z <- log10(out_chl$z)

oldPar <- par(no.readonly = FALSE)

par(mfrow = c(2, 2), mar = rep(0, 4), oma = rep(0, 4))

image(exampleSSTData, col = colPalette, axes = FALSE)
mtext(text = "Original SST", side = 3, line = -2, adj = 0.99, cex = 1.2)

image(out_sst, col = colPalette, axes = FALSE)
mtext(text = "SST gradient", side = 3, line = -2, adj = 0.99, cex = 1.2)

image(exampleChlData, col = colPalette, axes = FALSE)
mtext(text = "Original Chlorophyll", side = 3, line = -2, adj = 0.99, cex = 1.2)

image(out_chl, col = colPalette, axes = FALSE)
mtext(text = "Chlorophyll gradient\n(log scale)", side = 3, line = -4, adj = 0.99,
      cex = 1.2)

par(oldPar)
}
\references{
Belkin, I. M., & O'Reilly, J. E. (2009). An algorithm for oceanic
front detection in chlorophyll and SST satellite imagery. Journal of Marine
Systems, 78(3), 319-326 (\doi{10.1016/j.jmarsys.2008.11.018}).

Agenbag, J.J., A.J. Richardson, H. Demarcq, P. Freon, S. Weeks,
and F.A. Shillington. "Estimating Environmental Preferences of South African
Pelagic Fish Species Using Catch Size- and Remote Sensing Data". Progress in
Oceanography 59, No 2-3 (October 2003): 275-300.
(\doi{https://doi.org/10.1016/j.pocean.2003.07.004}).
}
